/************************************************************
*   Mega8 -  , 	*
* (c) 2011                              	*
* :                                   	*
*															*
*     							*
************************************************************/

/** @file uarc.c
 *  @brief      uart.c.
 */

/********************************************************************************************
*   (     .. 					*
*   .  15     					*
* 0x31 0x35  .																			*
*      uart1.c:                                   					*	
* :                                                 				   					*
* (AA)||||| |||||| 	*
* :																					*
* (AA)||||| +1||||||	*
********************************************************************************************/

#include "includes.h"

/******************************************************************************/
/*                                                                  */
/******************************************************************************/
uart_t  uart;


/******************************************************************************/
/*                                                                     */
/******************************************************************************/
/** @brief     .   
 *  @param                  
 *  @return 
 */
#pragma vector= USART_RXC_vect
__interrupt void  USART_RXC(void)
{
   uart.rx_buf[uart.rx_cnt]  = UDR;     //   
   uart.rx_cnt++;    					//     
   uart.state  = RECIEVER;   
   
   if (uart.rx_cnt >= uart.rx_buf[0])  
   {  //   
      UCSRB &=  ~((1<<RXEN ) | (1<<RXCIE ));   //  
      Put_cmd_in_buff(RX_COMPLETE,0);           //                       
      return;
   }
}

#pragma vector= USART_TXC_vect
__interrupt void  USART_TXC(void)
{
   uart.tx_cnt++;
   if (uart.tx_cnt >= uart.tx_buf[0]){
	  UCSRB &= ~((1<<TXEN ) | (1<<TXCIE ));  //  
      UCSRB |=  ((1<<RXEN ) | (1<<RXCIE ));   //       
      uart.rx_cnt = 0;
      uart.state  = WAIT_NEXT_PACKET;
      uart.rx_tmr = 0;
      return;
   }      
   
   UDR = uart.tx_buf[uart.tx_cnt];   
}


/** @brief   .   
 *  @param                  
 *  @return 
*/
void UartTimer (void)
{
   uart.rx_tmr++;
   if (uart.rx_tmr >= MAX_TIMEOUT){
      //          - 
      UCSRB &=  ~((1<<RXEN ) | (1<<RXCIE ));   //  
      uart.rx_tmr = 0;    
      uart.rx_cnt = 0;
      uart.state  = WAIT_NEXT_PACKET;
      UCSRB |=  ((1<<RXEN ) | (1<<RXCIE ));   //   
   }      
}


/** @brief      
 *  @param                  
 *  @return 
*/
void RxCompete (void)
{
	/*        */
	if (uart.rx_buf[1] == OWN_ADDRESS) {
		/*    */
		if (CaclculateCRC (uart.rx_buf)	== (uart.rx_buf[uart.rx_buf[0]-1])) {
			/*    -   */
			switch (uart.rx_buf[2]) {
				
				case SET_NEW_STATE: /*      */
					{
					/*   .   .
					 *    */
					unsigned int new_servo_state;
	
					/*      .
					 *      180       */
					new_servo_state = TIME_2_3ms - TIME_0_7ms;
	
					/*       */
					new_servo_state = new_servo_state / 180;
	
					/*    .    (- )
					 *              */
					new_servo_state = new_servo_state * uart.rx_buf[3] + TIME_0_7ms;
	
					Servo[0].timer = new_servo_state;
					
					/*     */
					uart.tx_buf[2] = SET_NEW_STATE_ANSWER;
					}
				break;
				
				default:
					return;
				break;
			}
		}
	    else {
			return;
		}				
	}
	else {
		return;
	}
   
	uart.state = TRANSMITER;
	uart.tx_cnt= 0;
   
	uart.tx_buf[0] = 5;
	uart.tx_buf[1] = MASTER_ADDRESS;   
	uart.tx_buf[3] = 0;
	uart.tx_buf[4] = CaclculateCRC (uart.tx_buf);

   	/*         */
	UCSRB =  ((1<<TXEN ) | (1<<TXCIE ));       
	UDR   = uart.tx_buf[0];
}


/** @brief      
 *  @param                  
 *  @return 
*/
unsigned char CaclculateCRC (unsigned char* buf)
{
   char j=0;
   for (char i=0; i < (buf[0]-1); i++){
      j += buf[i];
   }
   return (j);
}

/** @brief   
 *  @param                  
 *  @return 
*/
void UartInit (void)
{
    unsigned int UBRR_tmp;
    //   
    UBRR_tmp = (int)(((CPU_OSC*1000000)/ 16)/115200 - 1);
    UBRRH = (char)(UBRR_tmp >> 8 ); //   
    UBRRL = (char) UBRR_tmp;	    //   
	
    //  ,     transmit complete  receive complete
//    UCSRB =  ((1<<TXEN ) | (1<<RXEN ) | (1<<TXCIE ) | (1<<RXCIE ));    
	//  ,   receive complete
	UCSRB =  ((1<<RXEN ) | (1<<RXCIE ));    

   uart.rx_cnt = 0;
   uart.state  = WAIT_NEXT_PACKET;
   uart.rx_tmr = 0;
}